import pandas as pd
import seaborn as sb
import glob
import scipy as sp
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import itertools as it
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42
matplotlib.rcParams['font.size'] = 16
palette = "Set1"


def load(files):
    li = []
    all_files = glob.glob(files)
    print(len(all_files))
    for filename in all_files:
        # print(filename)
        df = pd.read_csv(filename)
        ogfilename = filename[0:-12] + "OG" + filename[-12:]
        # print(ogfilename)
        df.columns = [col.strip() for col in df.columns]
        odf = pd.read_csv(ogfilename)
        odf.columns = [col.strip() for col in odf.columns]
        # print(df.columns)
        if "hash_performance_3" in df.columns:
            # print("FOUND")
            df = df.rename(columns={"hash_performance_3": "hash_performance"})
            odf = odf.rename(columns={"hash_performance_3": "hash_performance"})
        elif "hash_performance_0" in df.columns:
            df = df.rename(columns={"hash_performance_0": "hash_performance"})
            odf = odf.rename(columns={"hash_performance_0": "hash_performance"})
        print(df.columns)
        # get starter function values (multiple in some cases)
        if df.at[1, "hash_id"] == 0:
            starter_fitness = min(df.at[0, "hash_fitness"], df.at[1, "hash_fitness"])
            starter_performance = min(df.at[0, "hash_performance"], df.at[1, "hash_performance"])
        else:
            starter_fitness = df.at[0, "hash_fitness"]
            starter_performance = df.at[0, "hash_performance"]
        original_fitness = odf.at[0, "hash_fitness"]
        original_performance = odf.at[0, "hash_performance"]
        # remove original function from frame
        df.drop(0, axis=0)
        # add relative fitness (with group by to match metrics)
        df["rel_fitness"] = df.hash_fitness / starter_fitness
        # add relative performance (with group by to match metrics)
        df["rel_performance"] = df.hash_performance / starter_performance
        # add og_rel_fitness
        df["og_rel_fitness"] = df.hash_fitness / original_fitness
        # add og_rel_performance
        df["og_rel_performance"] = df.hash_performance / original_performance
        # print(df.columns)
        means = df.groupby('generation').mean()['rel_fitness']
        # print(means)
        best = df.iloc[df.groupby(['generation'])['rel_fitness'].idxmin()]
        best['pop_mean'] = means
        means = df.groupby('generation').mean()['og_rel_fitness']
        # print(means)
        best['og_pop_mean'] = means
        li.append(best)
    frame = pd.concat(li, axis=0, ignore_index=True)
    return frame


# noinspection DuplicatedCode
def av_fit(data, pallet, label, name, title_list, og=False):
    # graph of the average fitness over time of the best individuals in each population
    sb.set(font_scale=1.125)
    data_property = ''
    data_property_full = ''
    if name == 'Best':
        data_property = "rel_fitness"
        data_property_full = "Relative Fitness"
    elif name == 'All':
        data_property = "pop_mean"
        data_property_full = "Mean Population Fitness"
    elif name == 'Perf':
        data_property = 'rel_performance'
        data_property_full = "Relative Performance"
    if og:
        data_property = "og_" + data_property
        data_property_full = "Original " + data_property_full
    if name not in ['Best', 'All']:
        data = data.loc[data[data_property] > 0]
    # Print average end values and p values of distribution comparisons
    final_gens = []
    for run in pd.unique(data.run_name):
        print("Final pop average for: " + label + " " + name + " " + run)
        final_gen = data.loc[(data['generation'] == 190) & (data['run_name'] == run)]
        print(np.mean(final_gen[data_property]))
        final_gens.append(final_gen)
    print("P values for: " + label)
    for pair in it.product(final_gens, repeat=2):
        print(pd.unique(pair[0].run_name) + " & " + pd.unique(pair[1].run_name))
        _, p = sp.stats.mannwhitneyu(pair[0][data_property], pair[1][data_property], use_continuity=False)
        print(p)
    # Create graph of data
    grid = sb.FacetGrid(data=data, row="run_name", hue="run_name", palette=pallet)
    grid.map(sb.lineplot, "generation", data_property, estimator="mean")
    axes = grid.axes.flatten()
    tn = 0
    for t in title_list:
        axes[tn].set_title(t)
        tn = tn + 1
    for ax in axes:
        ax.set_ylabel(data_property_full)
        ax.set_xlabel("Generation")
    # Display and save graph
    plt.savefig(label + name + "FitAv.png", bbox_inches='tight')
    plt.show()


def lineage_load(files, labels):
    li = []
    all_files = glob.glob(files)
    print(len(all_files))
    for filename in all_files:
        # print(filename)
        df = pd.read_csv(filename)
        df.columns = [col.strip() for col in df.columns]
        df["lineage_labels"] = [labels[x] for x in df["lineage"]]
        li.append(df)
    frame = pd.concat(li, axis=0, ignore_index=True)
    return frame


def lineage_extinction_times(mp, label):
    print("Extinction time: " + label)
    df_pivot = pd.pivot_table(data=mp, values="run_name", index="generation", columns="lineage_labels", aggfunc='count')
    for col in df_pivot.columns:
        df_pivot[col] = [0 if pd.isnull(x) else x for x in df_pivot[col]]
        df_pivot[col] = df_pivot[col]/900
    # print(df_pivot)
    print(df_pivot.loc[df_pivot[label] == 0])


def lineage_graph(mp, pal, label):
    # print(mp.columns)
    # counts = mp.groupby("generation", "lineage_labels").transform('count')
    df_pivot = pd.pivot_table(data=mp, values="run_name", index="generation", columns="lineage_labels", aggfunc='count')
    # print(df_pivot)
    for col in df_pivot.columns:
        df_pivot[col] = [0 if pd.isnull(x) else x for x in df_pivot[col]]
        df_pivot[col] = df_pivot[col]/900
    # print(df_pivot)
    g = sb.lineplot(data=df_pivot, palette=pal)
    g.set(xlabel="Generation", ylabel="Mean Proportion of Population")
    plt.legend(title="Lineage")
    plt.savefig(label + "LineageBalance.png", bbox_inches='tight')
    plt.show()
    pass


if __name__ == '__main__':
    # Load data in and add relative fitness and performance columns and remove the id 0 individuals
    # English Original Comparison
    # folder = "output_files/MPruns/*English/*/phylolog.csv"
    # fullData = load(folder)
    # labelName = "EnglishMPOG"
    # pal_type = sb.color_palette("Set3", 3)
    # title_list = ["English LCA", "Mixed Pop", "Polish LCA"]
    # av_fit(fullData, pal_type, labelName, "Best", title_list, og=True)
    # av_fit(fullData, pal_type, labelName, "Perf", title_list, og=True)
    # Polish Original Comparison
    # folder = "output_files/MPruns/*Polish/*/phylolog.csv"
    # fullData = load(folder)
    # labelName = "PolishMPOG"
    # pal_type = sb.color_palette("Set3", 3)
    # title_list = ["English LCA", "Mixed Pop", "Polish LCA"]
    # av_fit(fullData, pal_type, labelName, "Best", title_list, og=True)
    # av_fit(fullData, pal_type, labelName, "Perf", title_list, og=True)
    # English Source Comparison
    # folder = "output_files/MPruns/*English/*/phylolog.csv"
    # fullData = load(folder)
    # labelName = "EnglishMP"
    # pal_type = sb.color_palette("Set3", 3)
    # title_list = ["English LCA", "Mixed Pop", "Polish LCA"]
    # av_fit(fullData, pal_type, labelName, "Best", title_list)
    # av_fit(fullData, pal_type, labelName, "Perf", title_list)
    # Polish Source Comparison
    # folder = "output_files/MPruns/*Polish/*/phylolog.csv"
    # fullData = load(folder)
    # labelName = "PolishMP"
    # pal_type = sb.color_palette("Set3", 3)
    # title_list = ["English LCA", "Mixed Pop", "Polish LCA"]
    # av_fit(fullData, pal_type, labelName, "Best", title_list)
    # av_fit(fullData, pal_type, labelName, "Perf", title_list)
    # French Original Comparison
    # folder = "output_files/MPruns/*French/*/phylolog.csv"
    # fullData = load(folder)
    # labelName = "FrenchMPOG"
    # pal_type = sb.color_palette("Set3", 5)
    # title_list = ["English Finalist", "English LCA", "Mixed Pop", "Polish Finalist", "Polish LCA"]
    # av_fit(fullData, pal_type, labelName, "Best", title_list, og=True)
    # av_fit(fullData, pal_type, labelName, "Perf", title_list, og=True)

    # load each of the MP files
    # mp_eng_files = "output_files/MPruns/MPEnglish/*/phylolog.csv"
    # mp_pol_files = "output_files/MPruns/MPPolish/*/phylolog.csv"
    mp_fre_files = "output_files/MPruns/MPFrench/*/phylolog.csv"
    # create better named column with formatted lineage names
    # labels dictionary
    lineage_labels = {" lca/economist/Econ4.txt": "English", " lca/polish/Polish7.txt": "Polish"}
    # mp_eng = lineage_load(mp_eng_files, lineage_labels)
    # mp_pol = lineage_load(mp_pol_files, lineage_labels)
    mp_fre = lineage_load(mp_fre_files, lineage_labels)
    # Find and print the time of the first lineage extinction for each
    # lineage_extinction_times(mp_eng, "Polish")
    # lineage_extinction_times(mp_pol, "English")
    lineage_extinction_times(mp_fre, "English")
    lineage_extinction_times(mp_fre, "Polish")
    # # Graph of count/pop size of lineage names in each generation, averaged over runs
    pal_type = sb.color_palette("PuOr", 2)
    # lineage_graph(mp_eng, pal_type, "EnglishMP")
    # lineage_graph(mp_pol, pal_type, "PolishMP")
    lineage_graph(mp_fre, pal_type, "FrenchMP")
