import networkx as nx
import pandas as pd
import matplotlib.pyplot as plt
import pydot
from networkx.drawing.nx_pydot import graphviz_layout


class Node:
    def __init__(self, node_id, fit, level, ancestor):
        self.id = node_id
        self.fitness = fit
        self.generation = level
        self.parent = ancestor


def generate_tree(file):
    # read in dataframe
    df = pd.read_csv(file)
    # dictionary of nodes for reference
    nodes = {}
    # Make a directed graph object
    graph = nx.DiGraph()
    leaves = []
    # step through the dataframe
    for row in df.itertuples():
        # print(row)
        # check for node in dictionary and amend generation if lower in row
        if row.hash_id in nodes.keys():
            if row.generation < nodes[row.hash_id].generation:
                nodes[row.hash_id].generation = row.generation
        # create node object with level, fitness and ancestor id
        # correct ancestorsList values as coming from string
        node = Node(row.hash_id, row.rel_fitness, row.generation, int(row.hash_ancestorsList[2:-1]))
        # add node to dictionary with node id containing the node object
        nodes[node.id] = node
        # add node to graph
        # graph.add_node(node)
        if row.generation == 199:
            leaves.append(node)
    # Make list of edges
    edges = []
    # Run through node ids in dictionary and create list of node object - node object links
    for key in nodes.keys():
        child = nodes[key]
        parent = nodes[nodes[key].parent]
        edges.append((parent, child))
    # add edges to graph
    graph.add_edges_from(edges[1:])
    return graph, leaves


# display full tree with levels
def display_tree(graph, name, labels=None):
    print(graph)
    # nx.nx_agraph.write_dot(graph, 'test.dot')
    pos = nx.nx_agraph.graphviz_layout(graph, prog="dot", args='')
    if labels:
        nx.draw(graph, pos, node_size=20, node_color="blue", labels=labels, arrows=False)
    else:
        nx.draw(graph, pos, node_size=5, node_color="blue", with_labels=False, arrows=False)
    plt.savefig(name+".png", bbox_inches='tight')
    plt.show()


# return reduced tree for extant lines only look into existing functions
def extant_subtree(graph, extant):
    subgraph = nx.DiGraph()
    # find leaf nodes then work up the graph, should be a single edge in which the node is a child
    print(len(extant))
    for leaf in extant:
        current_child = leaf
        edge = [x for x in graph.edges if x[1] == current_child]
        while len(edge) > 0:
            edge = edge[0]
            subgraph.add_edges_from([edge])
            current_child = edge[0]
            edge = [x for x in graph.edges if x[1] == current_child]
    # get routes to root for each end node
    return subgraph


# Press the green button in the gutter to run the script.
def lca_subtree(subgraph):
    root = [n for n, d in subgraph.in_degree() if d == 0][0]
    children = [child[1] for child in subgraph.edges if child[0] == root]
    while len(children) < 2:
        root = children[0]
        children = [child[1] for child in subgraph.edges if child[0] == root]
    lca = nx.DiGraph()
    edges = [edge for edge in subgraph.edges if edge[0] == root]
    nodes = [edge[1] for edge in edges]
    lca.add_edges_from(edges)
    labels = {root: root.id}
    while len(edges) > 0:
        edges = []
        for node in nodes:
            edges = edges + [edge for edge in subgraph.edges if edge[0] == node]
        nodes = [edge[1] for edge in edges]
        for node in nodes:
            labels[node] = node.id
        lca.add_edges_from(edges)
    return lca, labels


if __name__ == '__main__':
    # filename = "output_files/Econ4_rankedphylolog.csv"
    # tree, leaves = generate_tree(filename)
    # # display_tree(tree, "fullTree_Econ4")
    # subtree = extant_subtree(tree, leaves)
    # # display_tree(subtree, "subTree_Econ4")
    # lcatree, lcalabels = lca_subtree(subtree)
    # # display_tree(lcatree, "lcaTree_Econ4_labeled", lcalabels)
    # # display_tree(lcatree, "lcaTree_Econ4")
    filename = "output_files/Polish7_rankedphylolog.csv"
    tree, leaves = generate_tree(filename)
    display_tree(tree, "fullTree_Polish7")
    subtree = extant_subtree(tree, leaves)
    display_tree(subtree, "subTree_Polish7")
    lcatree, lcalabels = lca_subtree(subtree)
    # display_tree(lcatree, "lcaTree_Polish7_labeled", lcalabels)
    display_tree(lcatree, "lcaTree_Polish7")
